@@ -5,10 +5,11 @@ import itertools
 
 import numpy as np
 import numba
-import scipy.spatial
+from scipy.spatial.distance import cdist
+from scipy.spatial import KDTree
 
-from . import _nearest_neighbours
-from . import utils
+from ._nns import generate_concentric_cloud
+from .utils import diameter
 
 
 def reconstruct(pdd, cell):
@@ -44,7 +45,7 @@ def reconstruct(pdd, cell):
     if dims not in (2, 3):
         raise ValueError('Reconstructing from PDD only implemented for 2 and 3 dimensions')
 
-    diam = utils.diameter(cell)
+    diam = diameter(cell)
     motif = [np.zeros((dims, ))]    # set first point as origin wlog, return if 1 motif point
 
     if pdd.shape[0] == 1:
@@ -52,7 +53,7 @@ def reconstruct(pdd, cell):
         return motif
 
     # finding lattice distances so we can ignore them
-    cloud_generator = _nearest_neighbours.generate_concentric_cloud(np.array(motif), cell)
+    cloud_generator = generate_concentric_cloud(np.array(motif), cell)
     cloud = []
     next(cloud_generator)
     l = next(cloud_generator)
@@ -168,11 +169,9 @@ def _neighbour_set(cell, prec):
     vecs = np.array(vecs)
 
     origin = np.zeros((1, cell.shape[0]))
-    cloud_generator = _nearest_neighbours.generate_concentric_cloud(origin, cell)
+    cloud_generator = generate_concentric_cloud(origin, cell)
     cloud = np.concatenate((next(cloud_generator), next(cloud_generator)))
-    tree = scipy.spatial.KDTree(cloud,
-                                compact_nodes=False,
-                                balanced_tree=False)
+    tree = KDTree(cloud, compact_nodes=False, balanced_tree=False)
     dists, inds = tree.query(vecs, k=k_, workers=-1)
     dists_ = np.empty_like(dists)
 
@@ -181,9 +180,7 @@ def _neighbour_set(cell, prec):
         cloud = np.vstack((cloud,
                            next(cloud_generator),
                            next(cloud_generator)))
-        tree = scipy.spatial.KDTree(cloud,
-                                    compact_nodes=False,
-                                    balanced_tree=False)
+        tree = KDTree(cloud, compact_nodes=False, balanced_tree=False)
         dists_, inds = tree.query(vecs, k=k_, workers=-1)
 
     tmp_inds = np.unique(inds[:, 1:].flatten())
@@ -197,7 +194,7 @@ def _neighbour_set(cell, prec):
     # so, check if the dist to 0 is leq (within tol) than dist to all other lattice points
     nn_norms = np.linalg.norm(neighbour_set, axis=-1)
     halves_norms = nn_norms / 2
-    halves_to_lattice_dists = scipy.spatial.distance.cdist(neighbour_set_half, neighbour_set)
+    halves_to_lattice_dists = cdist(neighbour_set_half, neighbour_set)
 
     # Do I need to + PREC?
     # inds of voronoi neighbours in cloud
