@@ -2,8 +2,10 @@
 cell parameters to Cartesian form, and an ETA class."""
 
 from typing import Tuple
+from amd.periodicset import PeriodicSet
 
 import numpy as np
+import numba
 from scipy.spatial.distance import squareform
 
 
@@ -25,17 +27,17 @@ def diameter(cell):
         ])
         d =  np.amax(np.linalg.norm(diams, axis=-1))
     else:
-        raise ValueError(f'diameter only implimented for dimensions <= 3 (passed {dims})')
+        raise ValueError(f'diameter only implimented for dimensions <= 3.')
     return d
 
 
+@numba.njit()
 def cellpar_to_cell(a, b, c, alpha, beta, gamma):
     """Simplified version of function from :mod:`ase.geometry` of the same name.
     3D unit cell parameters a,b,c,α,β,γ --> cell as 3x3 NumPy array.
     """
 
-    # Handle orthorhombic cells separately to avoid rounding errors
-    eps = 2 * np.spacing(90.0, dtype=np.float64)  # around 1.4e-14
+    eps = 2 * np.spacing(90.0)
 
     cos_alpha = 0. if abs(abs(alpha) - 90.) < eps else np.cos(alpha * np.pi / 180.)
     cos_beta = 0. if abs(abs(beta) - 90.) < eps else np.cos(beta * np.pi / 180.)
@@ -51,19 +53,29 @@ def cellpar_to_cell(a, b, c, alpha, beta, gamma):
     cy = (cos_alpha - cos_beta * cos_gamma) / sin_gamma
     cz_sqr = 1. - cos_beta ** 2 - cy ** 2
     if cz_sqr < 0:
-        raise RuntimeError('Could not create unit cell from parameters ' + \
-                           f'a={a},b={b},c={c},α={alpha},β={beta},γ={gamma}')
+        raise RuntimeError('Could not create unit cell from given parameters.')
 
-    return np.array([[a, 0, 0],
-                     [b*cos_gamma, b*sin_gamma, 0],
-                     [c*cos_beta, c*cy, c*np.sqrt(cz_sqr)]])
+    cell = np.zeros((3, 3))
+    cell[0, 0] = a
+    cell[1, 0] = b * cos_gamma
+    cell[1, 1] = b * sin_gamma
+    cell[2, 0] = c * cos_beta
+    cell[2, 1] = c * cy
+    cell[2, 2] = c * np.sqrt(cz_sqr)
+
+    return cell
 
 
+@numba.njit()
 def cellpar_to_cell_2D(a, b, alpha):
     """2D unit cell parameters a,b,α --> cell as 2x2 ndarray."""
+    
+    cell = np.zeros((2, 2))
+    cell[0, 0] = a
+    cell[1, 0] = b * np.cos(alpha * np.pi / 180.)
+    cell[1, 1] = b * np.sin(alpha * np.pi / 180.)
 
-    return np.array([[a, 0],
-                     [b * np.cos(alpha * np.pi / 180.), b * np.sin(alpha * np.pi / 180.)]])
+    return cell
 
 
 def neighbours_from_distance_matrix(
@@ -117,15 +129,22 @@ def neighbours_from_distance_matrix(
 
 
 def lattice_cubic(scale=1, dims=3):
-    """Return a pair ``(motif, cell)`` representing a cubic lattice, passable to
-    ``amd.AMD()`` or ``amd.PDD()``."""
+    """Return a pair ``(motif, cell)`` representing a hexagonal lattice, passable to
+    ``amd.AMD()`` or ``amd.PDD()``. """
 
-    return (np.zeros((1, dims)), np.identity(dims) * scale)
+    return PeriodicSet(np.zeros((1, dims)), np.identity(dims) * scale)
+
+
+def lattice_hexagonal(scale=1):
+    """Dimension 3 only. Return a pair ``(motif, cell)`` representing a cubic lattice, 
+    passable to ``amd.AMD()`` or ``amd.PDD()``."""
+
+    return PeriodicSet(np.zeros((1, 3)), cellpar_to_cell(scale, scale, scale, 90, 90, 120))
 
 
 def random_cell(length_bounds=(1, 2), angle_bounds=(60, 120), dims=3):
-    """Random unit cell with uniformally chosen length and angle parameters
-    between bounds."""
+    """Dimensions 2 and 3 only. Random unit cell with uniformally chosen length and 
+    angle parameters between bounds."""
 
     if dims == 3:
         while True:
